import { world, system,BlockVolume } from "@minecraft/server";
import { MessageFormData } from "@minecraft/server-ui";
import "./dekle_bhai_konsa_block_he.js"

const Build_wands = { "um:destroy_wand": { "MAX_BLOCKS": 2000 } };
const UNBREAKABLE_BLOCKS = [
    // 🚫 Completely Unbreakable Blocks
    "minecraft:bedrock",
    "minecraft:barrier",
    "minecraft:command_block",
    "minecraft:chain_command_block",
    "minecraft:repeating_command_block",
    "minecraft:structure_block",
    "minecraft:structure_void",
    "minecraft:deny",
    "minecraft:allow",
    "minecraft:jigsaw",

    // 🔥 Portal & Fluid Blocks
    "minecraft:end_portal",
    "minecraft:end_portal_frame",
    "minecraft:nether_portal",
    "minecraft:end_gateway",
    "minecraft:lava",
    "minecraft:flowing_lava",
    "minecraft:water",
    "minecraft:flowing_water",
    "minecraft:light_block",
    "minecraft:air",

    // ⏳ Extremely Hard Blocks (Takes a long time to break)
    "minecraft:obsidian", // ~10 seconds to break (Diamond Pickaxe)
    "minecraft:crying_obsidian", // ~10 seconds to break (Diamond Pickaxe)
     // ~82.5 seconds to break!
    "minecraft:ancient_debris", // Netherite source, very hard to break
    

    // 📦 Chests & Containers (To prevent item loss)
    "minecraft:chest",
    "minecraft:trapped_chest",
    "minecraft:barrel",
    "minecraft:shulker_box",
    "minecraft:ender_chest",
    "minecraft:furnace",
    "minecraft:blast_furnace",
    "minecraft:smoker",
    "minecraft:dispenser",
    "minecraft:dropper",
    "minecraft:hopper",
    "minecraft:beehive",
    "minecraft:bee_nest",

    // ⚠ Miscellaneous Hard Blocks
    "minecraft:netherite_block", // Very hard material
    "minecraft:crying_obsidian", // Special type of obsidian
    "minecraft:respawn_anchor", // Used in Nether for respawning
    "minecraft:lodestone" // Hard navigation block
];

world.afterEvents.itemUse.subscribe((event) => {
const player = event.source;
  const item = event.itemStack;
    let block = player?.getBlockFromViewDirection({ maxDistance: 15 }).block
    const loc = block.location;
    const inv = player.getComponent("inventory").container;
    
    

    if (!item || !Build_wands[item.typeId]) return;
    const gameMode = player.getGameMode?.(); 
const isCreative = gameMode === "creative";
let pickaxe;

if (isCreative) {
    pickaxe = false
    
} else {
  
  pickaxe = findPickaxe(inv, player.selectedSlotIndex);
    if (!pickaxe) return player.sendMessage("§cYou need a pickaxe in a nearby slot!");
}
    let lore = item.getLore() || [];
    lore.push(`${loc.x} ${loc.y} ${loc.z}`);
    item.setLore(lore);
    inv.setItem(player.selectedSlotIndex, item);

    player.runCommand(`particle minecraft:heart_particle ${loc.x} ${loc.y + 1} ${loc.z}`);
    player.playSound("random.orb");

    if (lore.length === 2) {
        destroyBlocks(player, lore, Build_wands[item.typeId].MAX_BLOCKS, pickaxe, inv);
        item.setLore([]);
        inv.setItem(player.selectedSlotIndex, item);
    }
});

function findPickaxe(inv, wandSlot) {
    const pickaxes = ["wooden", "stone", "iron", "golden", "diamond", "netherite"].map(type => `minecraft:${type}_pickaxe`);

    for (let offset of [-1, 1]) {
        let slotIndex = wandSlot + offset;
        let item = inv.getItem(slotIndex);
        if (item && pickaxes.includes(item.typeId)) return { item, slotIndex, durability: item.getComponent("durability") };
    }
    return null;
}

function destroyBlocks(player, positions, MAX_BLOCKS, pickaxe, inv) {
const gameMode = player.getGameMode?.(); 
const isCreative = gameMode === "creative";
const blocks = []
    const [pos1, pos2] = positions.map(pos => pos.split(" ").map(Number));
    if (isCreative) {
        blocks.push(0)
    } else{
    const [minX, minY, minZ] = pos1.map((v, i) => Math.min(v, pos2[i]));
    const [maxX, maxY, maxZ] = pos1.map((v, i) => Math.max(v, pos2[i]));

    
    for (let x = minX; x <= maxX; x++)
        for (let y = minY; y <= maxY; y++)
            for (let z = minZ; z <= maxZ; z++) {
                let block = player.dimension.getBlock({ x, y, z });
                if (block && block.typeId !== "minecraft:air" && !UNBREAKABLE_BLOCKS.includes(block.typeId)) blocks.push({ x, y, z });
            }

    if (!blocks.length) return player.sendMessage("§cNo valid blocks found!");

    if (blocks.length > MAX_BLOCKS) return player.sendMessage(`§cMax limit is ${MAX_BLOCKS} blocks!`);
    }
    

    
        new MessageFormData()
        .title("⚠ Confirm Block Destruction ⚠")
        .body(`Do you really want to destroy §e${blocks.length}§r blocks?\n\n§c⚠ This action cannot be undone! ⚠`)
        .button1("✅ Yes, Destroy")
        .button2("❌ No, Cancel")
        .show(player).then(res => {
            if (res.selection !== 0) return player.sendMessage("§cAction cancelled!");

            player.sendMessage(`§aDestroying ${blocks.length} blocks...`);
            player.playSound("random.explode");
            if (isCreative) {
            const volume = new BlockVolume({x: pos1[0], y: pos1[1] ,z: pos1[2]}, {x: pos2[0], y: pos2[1] ,z: pos2[2]});
    player.dimension.fillBlocks(volume, "minecraft:air");
      
            }else{
            let i = 0;
            function breakNext() {
                if (i >= blocks.length) {
                    player.sendMessage(`§aSuccessfully destroyed §e${blocks.length}§r blocks.`);
                    return player.playSound("block.anvil.destroy");
                }

                let { x, y, z } = blocks[i++];
                
                player.runCommand(`setblock ${x} ${y} ${z} air destroy`);
                player.runCommand(`particle minecraft:poof ${x} ${y} ${z}`);
                player.playSound("block.glass.break");
                
                
if (pickaxe.durability) {
let item = pickaxe.item.getComponent("minecraft:enchantable").getEnchantment("unbreaking")?.level
	
    if (pickaxe.durability.damage + 1 < pickaxe.durability.maxDurability) { 
    if(i % (item ? item +1 : 1) === 0){
        pickaxe.durability.damage += 1;
        }
        
       
        inv.setItem(pickaxe.slotIndex, pickaxe.item);
    } else {
        player.sendMessage("§eYour pickaxe is about to break! Stopping to prevent breakage.");
        return;
    }
}


                system.runTimeout(breakNext, 0.5);
            }
            breakNext();
            }
            
      
           });
     
}